# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-12 11:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('atris', '0007_related_field_history'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedhistoricalrecord',
            name='object_id',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalrecord',
            name='object_id',
            field=models.TextField(db_index=True),
        ),
    ]
