# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-02 09:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('atris', '0006_auto_20160622_0720'),
    ]

    operations = [
        migrations.AddField(
            model_name='archivedhistoricalrecord',
            name='related_field_history',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='referenced_objects_history', to='atris.ArchivedHistoricalRecord'),
        ),
        migrations.AddField(
            model_name='historicalrecord',
            name='related_field_history',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='referenced_objects_history', to='atris.HistoricalRecord'),
        ),
    ]
