# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2019-09-06 05:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('atris', '0008_object_id_text'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedhistoricalrecord',
            name='object_id',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='historicalrecord',
            name='object_id',
            field=models.TextField(),
        ),
        migrations.AlterIndexTogether(
            name='archivedhistoricalrecord',
            index_together=set([('object_id', 'history_date')]),
        ),
        migrations.AlterIndexTogether(
            name='historicalrecord',
            index_together=set([('object_id', 'history_date')]),
        ),
    ]
