# Generated by Django 4.2.6 on 2023-10-16 11:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('atris', '0009_auto_20190906_0509'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedhistoricalrecord',
            name='additional_data',
            field=models.JSONField(null=True),
        ),
        migrations.AlterField(
            model_name='archivedhistoricalrecord',
            name='data',
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name='historicalrecord',
            name='additional_data',
            field=models.JSONField(null=True),
        ),
        migrations.AlterField(
            model_name='historicalrecord',
            name='data',
            field=models.JSONField(),
        ),
    ]
